"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.module = void 0;
const tslib_1 = require("tslib");
const inversify_1 = require("inversify");
const base_1 = require("./base");
const cached_file_system_1 = require("../services/cached-file-system");
const ymir_1 = require("@fimbul/ymir");
const utils_1 = require("../utils");
const path = require("path");
let SaveCommandRunner = class SaveCommandRunner extends base_1.AbstractCommandRunner {
    constructor(fs, logger, directories, options) {
        super();
        this.fs = fs;
        this.logger = logger;
        this.directories = directories;
        this.options = options;
    }
    run({ command: _command, ...config }) {
        const newContent = utils_1.format({
            ...this.options,
            ...config,
            cache: config.cache || undefined,
            fix: config.fix || undefined,
            reportUselessDirectives: config.reportUselessDirectives || undefined,
            references: config.references || undefined,
        }, "yaml" /* Yaml */);
        const filePath = path.join(this.directories.getCurrentDirectory(), '.fimbullinter.yaml');
        if (newContent.trim() === '{}') {
            try {
                this.fs.remove(filePath);
                this.logger.log("Removed '.fimbullinter.yaml'.");
            }
            catch { }
        }
        else {
            this.fs.writeFile(filePath, newContent);
            this.logger.log("Updated '.fimbullinter.yaml'.");
        }
        return true;
    }
};
SaveCommandRunner = tslib_1.__decorate([
    inversify_1.injectable(),
    tslib_1.__metadata("design:paramtypes", [cached_file_system_1.CachedFileSystem,
        ymir_1.MessageHandler,
        ymir_1.DirectoryService,
        ymir_1.GlobalOptions])
], SaveCommandRunner);
exports.module = new inversify_1.ContainerModule((bind) => {
    bind(base_1.AbstractCommandRunner).to(SaveCommandRunner);
});
//# sourceMappingURL=save.js.map